@@ -19,30 +19,40 @@
 package org.apache.flink.connector.pulsar.sink;
 
 import org.apache.flink.annotation.PublicEvolving;
+import org.apache.flink.api.common.serialization.SerializationSchema;
 import org.apache.flink.configuration.ConfigOption;
 import org.apache.flink.configuration.Configuration;
 import org.apache.flink.connector.base.DeliveryGuarantee;
 import org.apache.flink.connector.pulsar.common.config.PulsarConfigBuilder;
 import org.apache.flink.connector.pulsar.common.config.PulsarOptions;
+import org.apache.flink.connector.pulsar.common.crypto.PulsarCrypto;
 import org.apache.flink.connector.pulsar.sink.config.SinkConfiguration;
 import org.apache.flink.connector.pulsar.sink.writer.delayer.MessageDelayer;
 import org.apache.flink.connector.pulsar.sink.writer.router.TopicRouter;
 import org.apache.flink.connector.pulsar.sink.writer.router.TopicRoutingMode;
 import org.apache.flink.connector.pulsar.sink.writer.serializer.PulsarSchemaWrapper;
 import org.apache.flink.connector.pulsar.sink.writer.serializer.PulsarSerializationSchema;
-import org.apache.flink.connector.pulsar.sink.writer.topic.TopicMetadataListener;
+import org.apache.flink.connector.pulsar.sink.writer.serializer.PulsarSerializationSchemaWrapper;
+import org.apache.flink.connector.pulsar.sink.writer.topic.MetadataListener;
 
+import org.apache.pulsar.client.api.ProducerCryptoFailureAction;
 import org.apache.pulsar.client.api.Schema;
+import org.apache.pulsar.common.schema.KeyValue;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.util.Arrays;
 import java.util.List;
+import java.util.Map;
 import java.util.Properties;
 
 import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_ADMIN_URL;
+import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_AUTH_PARAMS;
+import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_AUTH_PARAM_MAP;
+import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_AUTH_PLUGIN_CLASS_NAME;
 import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_ENABLE_TRANSACTION;
 import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_SERVICE_URL;
+import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_PRODUCER_CRYPTO_FAILURE_ACTION;
 import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_PRODUCER_NAME;
 import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_SEND_TIMEOUT_MS;
 import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_WRITE_DELIVERY_GUARANTEE;
@@ -50,6 +60,7 @@ import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_WR
 import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_WRITE_TRANSACTION_TIMEOUT;
 import static org.apache.flink.connector.pulsar.sink.config.PulsarSinkConfigUtils.SINK_CONFIG_VALIDATOR;
 import static org.apache.flink.connector.pulsar.source.enumerator.topic.TopicNameUtils.distinctTopics;
+import static org.apache.flink.util.InstantiationUtil.isSerializable;
 import static org.apache.flink.util.Preconditions.checkArgument;
 import static org.apache.flink.util.Preconditions.checkNotNull;
 import static org.apache.flink.util.Preconditions.checkState;
@@ -66,7 +77,7 @@ import static org.apache.flink.util.Preconditions.checkState;
  *     .setServiceUrl(operator().serviceUrl())
  *     .setAdminUrl(operator().adminUrl())
  *     .setTopics(topic)
- *     .setSerializationSchema(PulsarSerializationSchema.pulsarSchema(Schema.STRING))
+ *     .setSerializationSchema(Schema.STRING)
  *     .build();
  * }</pre>
  *
@@ -84,7 +95,7 @@ import static org.apache.flink.util.Preconditions.checkState;
  *     .setServiceUrl(operator().serviceUrl())
  *     .setAdminUrl(operator().adminUrl())
  *     .setTopics(topic)
- *     .setSerializationSchema(PulsarSerializationSchema.pulsarSchema(Schema.STRING))
+ *     .setSerializationSchema(Schema.STRING)
  *     .setDeliveryGuarantee(deliveryGuarantee)
  *     .build();
  * }</pre>
@@ -99,10 +110,11 @@ public class PulsarSinkBuilder<IN> {
     private final PulsarConfigBuilder configBuilder;
 
     private PulsarSerializationSchema<IN> serializationSchema;
-    private TopicMetadataListener metadataListener;
+    private MetadataListener metadataListener;
     private TopicRoutingMode topicRoutingMode;
     private TopicRouter<IN> topicRouter;
     private MessageDelayer<IN> messageDelayer;
+    private PulsarCrypto pulsarCrypto;
 
     // private builder constructor.
     PulsarSinkBuilder() {
@@ -162,7 +174,7 @@ public class PulsarSinkBuilder<IN> {
         checkState(metadataListener == null, "setTopics couldn't be set twice.");
         // Making sure the topic should be distinct.
         List<String> topicSet = distinctTopics(topics);
-        this.metadataListener = new TopicMetadataListener(topicSet);
+        this.metadataListener = new MetadataListener(topicSet);
         return this;
     }
 
@@ -208,6 +220,54 @@ public class PulsarSinkBuilder<IN> {
         return this;
     }
 
+    /**
+     * Send messages to Pulsar by using the flink's {@link SerializationSchema}. It would serialize
+     * the message into a byte array and send it to Pulsar with {@link Schema#BYTES}.
+     */
+    public <T extends IN> PulsarSinkBuilder<T> setSerializationSchema(
+            SerializationSchema<T> serializationSchema) {
+        return setSerializationSchema(new PulsarSerializationSchemaWrapper<>(serializationSchema));
+    }
+
+    /**
+     * Send messages to Pulsar by using the Pulsar {@link Schema} instance. It would serialize the
+     * message into a byte array and send it to Pulsar with {@link Schema#BYTES}. You can directly
+     * use the Schema you provided by enabling the {@link #enableSchemaEvolution()}.
+     *
+     * <p>We only support <a
+     * href="https://pulsar.apache.org/docs/en/schema-understand/#primitive-type">primitive
+     * types</a> here.
+     */
+    public <T extends IN> PulsarSinkBuilder<T> setSerializationSchema(Schema<T> schema) {
+        return setSerializationSchema(new PulsarSchemaWrapper<>(schema));
+    }
+
+    /**
+     * Send messages to Pulsar by using the Pulsar {@link Schema} instance. It would serialize the
+     * message into a byte array and send it to Pulsar with {@link Schema#BYTES}. You can directly
+     * use the Schema you provided by enabling the {@link #enableSchemaEvolution()}.
+     *
+     * <p>We only support <a
+     * href="https://pulsar.apache.org/docs/en/schema-understand/#struct">struct types</a> here.
+     */
+    public <T extends IN> PulsarSinkBuilder<T> setSerializationSchema(
+            Schema<T> schema, Class<T> typeClass) {
+        return setSerializationSchema(new PulsarSchemaWrapper<>(schema, typeClass));
+    }
+
+    /**
+     * Send messages to Pulsar by using the Pulsar {@link Schema} instance. It would serialize the
+     * message into a byte array and send it to Pulsar with {@link Schema#BYTES}. You can directly
+     * use the Schema you provided by enabling the {@link #enableSchemaEvolution()}.
+     *
+     * <p>We only support <a
+     * href="https://pulsar.apache.org/docs/en/schema-understand/#keyvalue">keyvalue types</a> here.
+     */
+    public <K, V, T extends IN> PulsarSinkBuilder<T> setSerializationSchema(
+            Schema<KeyValue<K, V>> schema, Class<K> keyClass, Class<V> valueClass) {
+        return setSerializationSchema(new PulsarSchemaWrapper<>(schema, keyClass, valueClass));
+    }
+
     /**
      * Sets the {@link PulsarSerializationSchema} that transforms incoming records to bytes.
      *
@@ -235,7 +295,8 @@ public class PulsarSinkBuilder<IN> {
     /**
      * Set a message delayer for enable Pulsar message delay delivery.
      *
-     * @param messageDelayer The delayer which would defined when to send the message to consumer.
+     * @param messageDelayer The delayer which would defined when to send the message to the
+     *     consumer.
      * @return this PulsarSinkBuilder.
      */
     public PulsarSinkBuilder<IN> delaySendingMessage(MessageDelayer<IN> messageDelayer) {
@@ -243,6 +304,49 @@ public class PulsarSinkBuilder<IN> {
         return this;
     }
 
+    /**
+     * Sets a {@link PulsarCrypto}. Configure the key reader and keys to be used to encrypt the
+     * message payloads.
+     *
+     * @param pulsarCrypto PulsarCrypto object.
+     * @return this PulsarSinkBuilder.
+     */
+    public PulsarSinkBuilder<IN> setPulsarCrypto(
+            PulsarCrypto pulsarCrypto, ProducerCryptoFailureAction action) {
+        this.pulsarCrypto = checkNotNull(pulsarCrypto);
+        configBuilder.set(PULSAR_PRODUCER_CRYPTO_FAILURE_ACTION, action);
+        return this;
+    }
+
+    /**
+     * Configure the authentication provider to use in the Pulsar client instance.
+     *
+     * @param authPluginClassName name of the Authentication-Plugin you want to use
+     * @param authParamsString string which represents parameters for the Authentication-Plugin,
+     *     e.g., "key1:val1,key2:val2"
+     * @return this PulsarSinkBuilder.
+     */
+    public PulsarSinkBuilder<IN> setAuthentication(
+            String authPluginClassName, String authParamsString) {
+        configBuilder.set(PULSAR_AUTH_PLUGIN_CLASS_NAME, authPluginClassName);
+        configBuilder.set(PULSAR_AUTH_PARAMS, authParamsString);
+        return this;
+    }
+
+    /**
+     * Configure the authentication provider to use in the Pulsar client instance.
+     *
+     * @param authPluginClassName name of the Authentication-Plugin you want to use
+     * @param authParams map which represents parameters for the Authentication-Plugin
+     * @return this PulsarSinkBuilder.
+     */
+    public PulsarSinkBuilder<IN> setAuthentication(
+            String authPluginClassName, Map<String, String> authParams) {
+        configBuilder.set(PULSAR_AUTH_PLUGIN_CLASS_NAME, authPluginClassName);
+        configBuilder.set(PULSAR_AUTH_PARAM_MAP, authParams);
+        return this;
+    }
+
     /**
      * Set an arbitrary property for the PulsarSink and Pulsar Producer. The valid keys can be found
      * in {@link PulsarSinkOptions} and {@link PulsarOptions}.
@@ -323,12 +427,16 @@ public class PulsarSinkBuilder<IN> {
         }
 
         checkNotNull(serializationSchema, "serializationSchema must be set.");
-        if (serializationSchema instanceof PulsarSchemaWrapper
-                && !Boolean.TRUE.equals(configBuilder.get(PULSAR_WRITE_SCHEMA_EVOLUTION))) {
+        // Schema evolution validation.
+        if (Boolean.TRUE.equals(configBuilder.get(PULSAR_WRITE_SCHEMA_EVOLUTION))) {
+            checkState(
+                    serializationSchema instanceof PulsarSchemaWrapper,
+                    "When enabling schema evolution, you must provide a Pulsar Schema in builder's setSerializationSchema method.");
+        } else if (serializationSchema instanceof PulsarSchemaWrapper) {
             LOG.info(
-                    "It seems like you want to send message in Pulsar Schema."
-                            + " You can enableSchemaEvolution for using this feature."
-                            + " We would use Schema.BYTES as the default schema if you don't enable this option.");
+                    "It seems like you are sending messages by using Pulsar Schema."
+                            + " You can builder.enableSchemaEvolution() to enable schema evolution for better Pulsar Schema check."
+                            + " We would use bypass Schema check by default.");
         }
 
         // Topic metadata listener validation.
@@ -339,11 +447,11 @@ public class PulsarSinkBuilder<IN> {
             } else {
                 LOG.warn(
                         "No topic set has been provided, make sure your custom topic router support empty topic set.");
-                this.metadataListener = new TopicMetadataListener();
+                this.metadataListener = new MetadataListener();
             }
         }
 
-        // Topic routing mode validate.
+        // Topic routing mode validation.
         if (topicRoutingMode == null) {
             LOG.info("No topic routing mode has been chosen. We use round-robin mode as default.");
             this.topicRoutingMode = TopicRoutingMode.ROUND_ROBIN;
@@ -353,6 +461,17 @@ public class PulsarSinkBuilder<IN> {
             this.messageDelayer = MessageDelayer.never();
         }
 
+        if (pulsarCrypto == null) {
+            this.pulsarCrypto = PulsarCrypto.disabled();
+        }
+
+        // Make sure they are serializable.
+        checkState(
+                isSerializable(serializationSchema),
+                "PulsarSerializationSchema isn't serializable");
+        checkState(isSerializable(messageDelayer), "MessageDelayer isn't serializable");
+        checkState(isSerializable(pulsarCrypto), "PulsarCrypto isn't serializable");
+
         // This is an unmodifiable configuration for Pulsar.
         // We don't use Pulsar's built-in configure classes for compatible requirement.
         SinkConfiguration sinkConfiguration =
@@ -364,12 +483,13 @@ public class PulsarSinkBuilder<IN> {
                 metadataListener,
                 topicRoutingMode,
                 topicRouter,
-                messageDelayer);
+                messageDelayer,
+                pulsarCrypto);
     }
 
     // ------------- private helpers  --------------
 
-    /** Helper method for java compiler recognize the generic type. */
+    /** Helper method for java compiler recognizes the generic type. */
     @SuppressWarnings("unchecked")
     private <T extends IN> PulsarSinkBuilder<T> specialized() {
         return (PulsarSinkBuilder<T>) this;
