@@ -22,37 +22,34 @@ import org.apache.flink.configuration.Configuration;
 import org.apache.flink.connector.base.DeliveryGuarantee;
 import org.apache.flink.connector.pulsar.common.config.PulsarConfiguration;
 import org.apache.flink.connector.pulsar.source.enumerator.topic.TopicPartition;
-import org.apache.flink.connector.testframe.external.ExternalContext;
-
-import org.apache.flink.shaded.guava30.com.google.common.base.Strings;
 
 import org.apache.pulsar.client.admin.PulsarAdmin;
 import org.apache.pulsar.client.admin.PulsarAdminException;
 import org.apache.pulsar.client.admin.PulsarAdminException.ConflictException;
 import org.apache.pulsar.client.admin.PulsarAdminException.NotFoundException;
 import org.apache.pulsar.client.api.Consumer;
-import org.apache.pulsar.client.api.ConsumerBuilder;
 import org.apache.pulsar.client.api.Message;
 import org.apache.pulsar.client.api.MessageId;
 import org.apache.pulsar.client.api.Producer;
-import org.apache.pulsar.client.api.ProducerBuilder;
 import org.apache.pulsar.client.api.PulsarClient;
-import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.client.api.TypedMessageBuilder;
 import org.apache.pulsar.client.api.transaction.TransactionCoordinatorClient;
 import org.apache.pulsar.client.api.transaction.TxnID;
-import org.apache.pulsar.client.impl.PulsarClientImpl;
+import org.apache.pulsar.common.naming.NamespaceName;
 import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.partition.PartitionedTopicMetadata;
+import org.apache.pulsar.common.policies.data.TenantInfo;
+import org.apache.pulsar.shade.com.google.common.base.Strings;
 
 import java.io.Closeable;
+import java.io.IOException;
 import java.time.Duration;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Random;
-import java.util.concurrent.ExecutionException;
 import java.util.function.Supplier;
 import java.util.stream.Stream;
 
@@ -65,9 +62,7 @@ import static org.apache.flink.connector.base.DeliveryGuarantee.EXACTLY_ONCE;
 import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_ADMIN_URL;
 import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_ENABLE_TRANSACTION;
 import static org.apache.flink.connector.pulsar.common.config.PulsarOptions.PULSAR_SERVICE_URL;
-import static org.apache.flink.connector.pulsar.common.utils.PulsarExceptionUtils.sneakyAdmin;
-import static org.apache.flink.connector.pulsar.common.utils.PulsarExceptionUtils.sneakyClient;
-import static org.apache.flink.connector.pulsar.common.utils.PulsarExceptionUtils.sneakyThrow;
+import static org.apache.flink.connector.pulsar.common.utils.PulsarTransactionUtils.getTcClient;
 import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_SEND_TIMEOUT_MS;
 import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_WRITE_DELIVERY_GUARANTEE;
 import static org.apache.flink.connector.pulsar.sink.PulsarSinkOptions.PULSAR_WRITE_TRANSACTION_TIMEOUT;
@@ -80,10 +75,7 @@ import static org.apache.pulsar.client.api.SubscriptionMode.Durable;
 import static org.apache.pulsar.client.api.SubscriptionType.Exclusive;
 import static org.apache.pulsar.common.partition.PartitionedTopicMetadata.NON_PARTITIONED;
 
-/**
- * A pulsar cluster operator used for operating pulsar instance. It's serializable for using in
- * {@link ExternalContext}.
- */
+/** A pulsar cluster operator used for operating pulsar instance. */
 public class PulsarRuntimeOperator implements Closeable {
 
     public static final int DEFAULT_PARTITIONS = 10;
@@ -95,7 +87,7 @@ public class PulsarRuntimeOperator implements Closeable {
     private final PulsarClient client;
     private final PulsarAdmin admin;
 
-    public PulsarRuntimeOperator(String serviceUrl, String adminUrl) {
+    public PulsarRuntimeOperator(String serviceUrl, String adminUrl) throws Exception {
         this(serviceUrl, serviceUrl, adminUrl, adminUrl);
     }
 
@@ -103,17 +95,35 @@ public class PulsarRuntimeOperator implements Closeable {
             String serviceUrl,
             String containerServiceUrl,
             String adminUrl,
-            String containerAdminUrl) {
+            String containerAdminUrl)
+            throws Exception {
         this.serviceUrl = containerServiceUrl;
         this.adminUrl = containerAdminUrl;
-        this.client =
-                sneakyClient(
-                        () ->
-                                PulsarClient.builder()
-                                        .serviceUrl(serviceUrl)
-                                        .enableTransaction(true)
-                                        .build());
-        this.admin = sneakyClient(() -> PulsarAdmin.builder().serviceHttpUrl(adminUrl).build());
+        this.client = PulsarClient.builder().serviceUrl(serviceUrl).enableTransaction(true).build();
+        this.admin = PulsarAdmin.builder().serviceHttpUrl(adminUrl).build();
+    }
+
+    /** Create a tenant if it's not existed in Pulsar. */
+    public void createTenant(String tenant) throws PulsarAdminException {
+        List<String> tenants = admin.tenants().getTenants();
+        if (!tenants.contains(tenant)) {
+            List<String> clusters = admin.clusters().getClusters();
+            TenantInfo tenantInfo =
+                    TenantInfo.builder().allowedClusters(new HashSet<>(clusters)).build();
+            admin.tenants().createTenant(tenant, tenantInfo);
+        }
+    }
+
+    /** Create a namespace if it's not existed in Pulsar. */
+    public void createNamespace(String namespace) throws PulsarAdminException {
+        NamespaceName namespaceName = NamespaceName.get(namespace);
+        String tenant = namespaceName.getTenant();
+
+        createTenant(tenant);
+        List<String> namespaces = admin.namespaces().getNamespaces(tenant);
+        if (!namespaces.contains(namespace)) {
+            admin.namespaces().createNamespace(namespace);
+        }
     }
 
     /**
@@ -122,7 +132,7 @@ public class PulsarRuntimeOperator implements Closeable {
      *
      * @param topic Pulsar topic name, it couldn't be a name with partition index.
      */
-    public void setupTopic(String topic) {
+    public void setupTopic(String topic) throws Exception {
         Random random = new Random(System.currentTimeMillis());
         setupTopic(topic, Schema.STRING, () -> randomAlphanumeric(10 + random.nextInt(20)));
     }
@@ -135,7 +145,8 @@ public class PulsarRuntimeOperator implements Closeable {
      * @param schema The Pulsar schema for serializing records into bytes.
      * @param supplier The supplier for providing the records which would be sent to Pulsar.
      */
-    public <T> void setupTopic(String topic, Schema<T> schema, Supplier<T> supplier) {
+    public <T> void setupTopic(String topic, Schema<T> schema, Supplier<T> supplier)
+            throws Exception {
         setupTopic(topic, schema, supplier, NUM_RECORDS_PER_PARTITION);
     }
 
@@ -149,7 +160,8 @@ public class PulsarRuntimeOperator implements Closeable {
      * @param numRecordsPerSplit The number of records for a partition.
      */
     public <T> void setupTopic(
-            String topic, Schema<T> schema, Supplier<T> supplier, int numRecordsPerSplit) {
+            String topic, Schema<T> schema, Supplier<T> supplier, int numRecordsPerSplit)
+            throws Exception {
         String topicName = topicName(topic);
         createTopic(topicName, DEFAULT_PARTITIONS);
 
@@ -171,7 +183,7 @@ public class PulsarRuntimeOperator implements Closeable {
      * @param numberOfPartitions The number of partitions. We would create a non-partitioned topic
      *     if this number is zero.
      */
-    public void createTopic(String topic, int numberOfPartitions) {
+    public void createTopic(String topic, int numberOfPartitions) throws Exception {
         checkArgument(numberOfPartitions >= 0);
         if (numberOfPartitions == 0) {
             createNonPartitionedTopic(topic);
@@ -180,8 +192,8 @@ public class PulsarRuntimeOperator implements Closeable {
         }
     }
 
-    public void createSchema(String topic, Schema<?> schema) {
-        sneakyAdmin(() -> admin().schemas().createSchema(topic, schema.getSchemaInfo()));
+    public void createSchema(String topic, Schema<?> schema) throws Exception {
+        admin().schemas().createSchema(topic, schema.getSchemaInfo());
     }
 
     /**
@@ -190,14 +202,13 @@ public class PulsarRuntimeOperator implements Closeable {
      * @param topic The topic name.
      * @param newPartitionsNum The new partition size which should exceed previous size.
      */
-    public void increaseTopicPartitions(String topic, int newPartitionsNum) {
-        PartitionedTopicMetadata metadata =
-                sneakyAdmin(() -> admin().topics().getPartitionedTopicMetadata(topic));
+    public void increaseTopicPartitions(String topic, int newPartitionsNum) throws Exception {
+        PartitionedTopicMetadata metadata = admin().topics().getPartitionedTopicMetadata(topic);
         checkArgument(
                 metadata.partitions < newPartitionsNum,
                 "The new partition size which should greater than previous size.");
 
-        sneakyAdmin(() -> admin().topics().updatePartitionedTopic(topic, newPartitionsNum));
+        admin().topics().updatePartitionedTopic(topic, newPartitionsNum);
     }
 
     /**
@@ -205,7 +216,7 @@ public class PulsarRuntimeOperator implements Closeable {
      *
      * @param topic The topic name.
      */
-    public void deleteTopic(String topic) {
+    public void deleteTopic(String topic) throws Exception {
         String topicName = topicName(topic);
         PartitionedTopicMetadata metadata;
 
@@ -214,27 +225,20 @@ public class PulsarRuntimeOperator implements Closeable {
         } catch (NotFoundException e) {
             // This topic doesn't exist. Just skip deletion.
             return;
-        } catch (PulsarAdminException e) {
-            sneakyThrow(e);
-            return;
         }
 
         if (metadata.partitions == NON_PARTITIONED) {
-            sneakyAdmin(() -> admin().topics().delete(topicName));
+            admin().topics().delete(topicName);
         } else {
-            sneakyAdmin(() -> admin().topics().deletePartitionedTopic(topicName));
+            admin().topics().deletePartitionedTopic(topicName);
         }
     }
 
     /** Convert the topic metadata into a list of topic partitions. */
-    public List<TopicPartition> topicInfo(String topic) {
-        try {
-            return client().getPartitionsForTopic(topic).get().stream()
-                    .map(p -> new TopicPartition(topic, TopicName.getPartitionIndex(p)))
-                    .collect(toList());
-        } catch (InterruptedException | ExecutionException e) {
-            throw new IllegalStateException(e);
-        }
+    public List<TopicPartition> topicInfo(String topic) throws Exception {
+        return client().getPartitionsForTopic(topic).get().stream()
+                .map(p -> new TopicPartition(topic, TopicName.getPartitionIndex(p)))
+                .collect(toList());
     }
 
     /**
@@ -246,7 +250,7 @@ public class PulsarRuntimeOperator implements Closeable {
      * @param <T> The type of the record.
      * @return message id.
      */
-    public <T> MessageId sendMessage(String topic, Schema<T> schema, T message) {
+    public <T> MessageId sendMessage(String topic, Schema<T> schema, T message) throws Exception {
         List<MessageId> messageIds = sendMessages(topic, schema, singletonList(message));
         checkArgument(messageIds.size() == 1);
 
@@ -263,7 +267,8 @@ public class PulsarRuntimeOperator implements Closeable {
      * @param <T> The type of the record.
      * @return message id.
      */
-    public <T> MessageId sendMessage(String topic, Schema<T> schema, String key, T message) {
+    public <T> MessageId sendMessage(String topic, Schema<T> schema, String key, T message)
+            throws Exception {
         List<MessageId> messageIds = sendMessages(topic, schema, key, singletonList(message));
         checkArgument(messageIds.size() == 1);
 
@@ -279,8 +284,8 @@ public class PulsarRuntimeOperator implements Closeable {
      * @param <T> The type of the record.
      * @return message id.
      */
-    public <T> List<MessageId> sendMessages(
-            String topic, Schema<T> schema, Collection<T> messages) {
+    public <T> List<MessageId> sendMessages(String topic, Schema<T> schema, Collection<T> messages)
+            throws Exception {
         return sendMessages(topic, schema, null, messages);
     }
 
@@ -295,10 +300,9 @@ public class PulsarRuntimeOperator implements Closeable {
      * @return message id.
      */
     public <T> List<MessageId> sendMessages(
-            String topic, Schema<T> schema, String key, Collection<T> messages) {
+            String topic, Schema<T> schema, String key, Collection<T> messages) throws Exception {
         try (Producer<T> producer = createProducer(topic, schema)) {
             List<MessageId> messageIds = new ArrayList<>(messages.size());
-
             for (T message : messages) {
                 TypedMessageBuilder<T> builder = producer.newMessage().value(message);
                 if (!Strings.isNullOrEmpty(key)) {
@@ -309,9 +313,6 @@ public class PulsarRuntimeOperator implements Closeable {
             }
             producer.flush();
             return messageIds;
-        } catch (PulsarClientException e) {
-            sneakyThrow(e);
-            return emptyList();
         }
     }
 
@@ -319,14 +320,11 @@ public class PulsarRuntimeOperator implements Closeable {
      * Consume a message from the given Pulsar topic, this method would be blocked until we get a
      * message from this topic.
      */
-    public <T> Message<T> receiveMessage(String topic, Schema<T> schema) {
+    public <T> Message<T> receiveMessage(String topic, Schema<T> schema) throws Exception {
         try (Consumer<T> consumer = createConsumer(topic, schema)) {
             Message<T> message = consumer.receive();
             consumer.acknowledge(message.getMessageId());
             return message;
-        } catch (PulsarClientException e) {
-            sneakyThrow(e);
-            return null;
         }
     }
 
@@ -339,7 +337,6 @@ public class PulsarRuntimeOperator implements Closeable {
             Message<T> message =
                     consumer.receive(Math.toIntExact(timeout.toMillis()), MILLISECONDS);
             consumer.acknowledge(message.getMessageId());
-
             return message;
         } catch (Exception e) {
             return null;
@@ -350,7 +347,8 @@ public class PulsarRuntimeOperator implements Closeable {
      * Consume a fixed number of messages from the given Pulsar topic, this method would be blocked
      * until we get the exactly number of messages from this topic.
      */
-    public <T> List<Message<T>> receiveMessages(String topic, Schema<T> schema, int counts) {
+    public <T> List<Message<T>> receiveMessages(String topic, Schema<T> schema, int counts)
+            throws Exception {
         if (counts == 0) {
             return emptyList();
         } else if (counts < 0) {
@@ -369,10 +367,8 @@ public class PulsarRuntimeOperator implements Closeable {
                     messages.add(message);
                     consumer.acknowledge(message.getMessageId());
                 }
+
                 return messages;
-            } catch (PulsarClientException e) {
-                sneakyThrow(e);
-                return emptyList();
             }
         }
     }
@@ -396,7 +392,7 @@ public class PulsarRuntimeOperator implements Closeable {
 
     /** Return the transaction coordinator client for operating {@link TxnID}. */
     public TransactionCoordinatorClient coordinatorClient() {
-        return ((PulsarClientImpl) client()).getTcClient();
+        return getTcClient(client());
     }
 
     /**
@@ -447,9 +443,12 @@ public class PulsarRuntimeOperator implements Closeable {
         return configuration;
     }
 
-    /** This method is used for test framework. You can't close this operator manually. */
+    /**
+     * This method is used for the connector test framework. You don't need to close this operator
+     * manually.
+     */
     @Override
-    public void close() throws PulsarClientException {
+    public void close() throws IOException {
         if (admin != null) {
             admin.close();
         }
@@ -460,55 +459,50 @@ public class PulsarRuntimeOperator implements Closeable {
 
     // --------------------------- Private Methods -----------------------------
 
-    private void createNonPartitionedTopic(String topic) {
+    private void createNonPartitionedTopic(String topic) throws Exception {
         try {
             admin().topics().createNonPartitionedTopic(topic);
         } catch (PulsarAdminException e) {
             if (!(e instanceof ConflictException
                     && e.getMessage().equals("This topic already exists"))) {
-                sneakyThrow(e);
+                throw e;
             }
         }
     }
 
-    private void createPartitionedTopic(String topic, int numberOfPartitions) {
+    private void createPartitionedTopic(String topic, int numberOfPartitions) throws Exception {
         try {
             admin().topics().createPartitionedTopic(topic, numberOfPartitions);
         } catch (PulsarAdminException e) {
             if (!(e instanceof ConflictException
                     && e.getMessage().equals("This topic already exists"))) {
-                sneakyThrow(e);
+                throw e;
             }
         }
     }
 
-    private <T> Producer<T> createProducer(String topic, Schema<T> schema) {
-        ProducerBuilder<T> builder =
-                client().newProducer(schema)
-                        .topic(topic)
-                        .enableBatching(false)
-                        .enableMultiSchema(true)
-                        .accessMode(Shared);
-
-        return sneakyClient(builder::create);
+    private <T> Producer<T> createProducer(String topic, Schema<T> schema) throws Exception {
+        return client().newProducer(schema)
+                .topic(topic)
+                .enableBatching(false)
+                .enableMultiSchema(true)
+                .accessMode(Shared)
+                .create();
     }
 
-    private <T> Consumer<T> createConsumer(String topic, Schema<T> schema) {
+    private <T> Consumer<T> createConsumer(String topic, Schema<T> schema) throws Exception {
         // Create the earliest subscription if it's not existed.
-        List<String> subscriptions = sneakyAdmin(() -> admin().topics().getSubscriptions(topic));
+        List<String> subscriptions = admin().topics().getSubscriptions(topic);
         if (!subscriptions.contains(SUBSCRIPTION_NAME)) {
-            sneakyAdmin(
-                    () -> admin().topics().createSubscription(topic, SUBSCRIPTION_NAME, earliest));
+            admin().topics().createSubscription(topic, SUBSCRIPTION_NAME, earliest);
         }
 
         // Create the consumer without the initial position.
-        ConsumerBuilder<T> builder =
-                client().newConsumer(schema)
-                        .topic(topic)
-                        .subscriptionName(SUBSCRIPTION_NAME)
-                        .subscriptionMode(Durable)
-                        .subscriptionType(Exclusive);
-
-        return sneakyClient(builder::subscribe);
+        return client().newConsumer(schema)
+                .topic(topic)
+                .subscriptionName(SUBSCRIPTION_NAME)
+                .subscriptionMode(Durable)
+                .subscriptionType(Exclusive)
+                .subscribe();
     }
 }
